package main

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"log"
	"net/http"

	calendar "google.golang.org/api/calendar/v3"

	"golang.org/x/oauth2"
	"golang.org/x/oauth2/google"
)

type CalClient struct {
	rawClientSecret []byte
	rawToken        []byte
	ctx             context.Context
	cfg             *oauth2.Config
	tkn             *oauth2.Token
	client          *http.Client
}

func CreateClient(secret, tkn []byte) *CalClient {
	var err error
	c := CalClient{
		rawClientSecret: []byte(secret),
		rawToken:        []byte(tkn),
	}

	c.ctx = context.Background()
	c.cfg, err = google.ConfigFromJSON(c.rawClientSecret, calendar.CalendarScope)

	if err != nil {
		log.Fatalf("unable to parse client secret file to config: %v", err)
	}

	// Try to create the client with the passed token
	c.client, err = c.GetClient()
	if err != nil {
		log.Fatalf("unable to create client: %v", err)
	}
	return &c
}

func (c *CalClient) GetClient() (*http.Client, error) {
	err := c.tokenFromRaw()
	if err != nil {
		err = c.tokenFromWeb()
		if err != nil {
			return nil, err
		}
	}
	return c.cfg.Client(c.ctx, c.tkn), nil
}

func (c *CalClient) tokenFromRaw() error {
	c.tkn = &oauth2.Token{}
	tmpToken := bytes.NewBuffer(c.rawToken)
	err := json.NewDecoder(tmpToken).Decode(c.tkn)
	return err
}

func (c *CalClient) tokenFromWeb() error {
	var err error
	authURL := c.cfg.AuthCodeURL("state-token", oauth2.AccessTypeOffline)
	fmt.Printf("Go to the following link in your browser then type the authorization code:\n%v\n", authURL)
	var code string
	if _, err := fmt.Scan(&code); err != nil {
		log.Fatalf("Unable to read authorization code %v", err)
	}

	c.tkn, err = c.cfg.Exchange(oauth2.NoContext, code)
	return err
}

func (c *CalClient) TokenToRaw() {
	var tmpToken bytes.Buffer
	t := c.tkn
	json.NewEncoder(&tmpToken).Encode(t)
	c.rawToken = tmpToken.Bytes()
}
